#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import re
import asyncio

from urllib.parse import urljoin

from .db import DB
from .wordlist import get_wordlist
from .data import PlecostWordPressInfo
from .utils import get_diff_ratio, update_progress, download, colorize

REGEX_FIND_VER = r'''(\?ver\=[\s]*)([\.\d\w]+)([\s\"\']+)'''
REGEX_FIND_CSS_SCRIPT_LINKS = r'''((script|link).*(src|href)=)(.*)(>)'''


# ----------------------------------------------------------------------
# WordPress testing functions
# ----------------------------------------------------------------------
@asyncio.coroutine
def is_remote_a_wordpress(base_url, error_page, downloader):
    """
    This functions checks if remote host contains a WordPress installation.

    :param base_url: Base url
    :type base_url: basestring

    :param error_page: error page content
    :type error_page: basestring

    :param downloader: download function. This function must accept only one parameter: the URL
    :type downloader: function

    :return: True if target contains WordPress installation. False otherwise.
    :rtype: bool
    """
    total_urls = 0
    urls_found = 0

    for url in update_progress(get_wordlist("wordpress_detection.txt"),
                               prefix_text="   "):
        total_urls += 1
        # Fix the url for urljoin
        path = url[1:] if url.startswith("/") else url

        headers, status, content = yield from downloader(urljoin(base_url, path))

        if status == 200:
            # Try to detect non-default error pages
            ratio = get_diff_ratio(content, error_page)
            if ratio < 0.35:
                urls_found += 1

    # Check that al least the 85% of typical Wordpress links exits
    if (urls_found / float(total_urls)) < 0.85:

        #
        # If the Wordpress Admin site is found --> Is a Wordpress Site
        #
        _headers, _status, _content = yield from downloader(urljoin(base_url, "/wp-admin/"))
        if _status == 302 and "wp-login.php?redirect_to=" in _headers.get("location", ""):
            return True
        elif _status == 301 and "/wp-admin/" in _headers.get("location", ""):
            return True
        elif _status == 200:
            return True
        else:

            #
            # Try to find wordpress Links
            #
            _, _, _content = yield from downloader(base_url)

            _web_links = re.findall(REGEX_FIND_CSS_SCRIPT_LINKS,
                                    _content)

            is_wp_content = any("/wp-content/" in x[3] for x in
                                _web_links)
            is_wp_includes = any("/wp-includes/" in x[3] for x in
                                 _web_links)

            if is_wp_content or is_wp_includes:
                return True
            else:
                return False

    else:
        return True


# ----------------------------------------------------------------------
@asyncio.coroutine
def get_wordpress_version(url, downloader, db):
    """
    This functions checks remote WordPress version.

    :param url: site to looking for WordPress version
    :type url: basestring

    :param downloader: download function. This function must accept only one parameter: the URL
    :type downloader: function

    :param db: cve database instance
    :type db: DB

    :return: PlecostWordPressInfo instance.
    :rtype: `PlecostWordPressInfo`
    """
    url_version = {
        # Generic
        "wp-login.php": r"(;ver=)([0-9\.]+)([\-a-z]*)",

        # For WordPress 3.8
        "wp-admin/css/wp-admin-rtl.css": r"(Version[\s]+)([0-9\.]+)",
        "wp-admin/css/wp-admin.css": r"(Version[\s]+)([0-9\.]+)"
    }

    # --------------------------------------------------------------------------
    # Get last Wordpress version
    # --------------------------------------------------------------------------

    # URL to get last version of WordPress available
    try:
        _, _, last_version_content = yield from downloader("https://wordpress.org/download/")

        last_version = re.search("(WordPress\&nbsp\;)([0-9\.]*)", str(last_version_content))
        if last_version is None:
            last_version = "unknown"
        else:
            if len(last_version.groups()) != 2:
                last_version = "unknown"
            else:
                last_version = last_version.group(2)
    except Exception:
        last_version = "unknown"

    # --------------------------------------------------------------------------
    #
    # Get installed version:
    #
    # --------------------------------------------------------------------------

    # --------------------------------------------------------------------------
    # Method 1: Looking for in readme.txt
    # --------------------------------------------------------------------------
    headers, status, curr_content = yield from downloader(urljoin(url, "/readme.html"))

    curr_ver = None
    if curr_content is not None:

        curr_ver = re.search(r"""(<br[\s]*/>[\s]*[Vv]ersion[\s]*)([\d]\.[\d]\.*[\d]*)""", curr_content)
        if curr_ver is None:
            curr_ver = None
        else:
            if len(curr_ver.groups()) != 2:
                curr_ver = None
            else:
                curr_ver = curr_ver.group(2)

    # --------------------------------------------------------------------------
    # Method 2: Looking for meta tag
    # --------------------------------------------------------------------------
    _, _, curr_content_2 = yield from downloader(url)

    # Try to find the info
    cur_ver_2 = None
    if curr_content_2 is not None:
        cur_ver_2 = re.search(r'''(<meta name=\"generator\" content=\"WordPress[\s]+)([0-9\.]+)''', curr_content_2)
        if cur_ver_2 is None:
            cur_ver_2 = None
        else:
            if len(cur_ver_2.groups()) != 2:
                cur_ver_2 = None
            else:
                cur_ver_2 = cur_ver_2.group(2)

    # --------------------------------------------------------------------------
    # Method 3: find in '?ver=' vars
    # --------------------------------------------------------------------------
    if not cur_ver_2:

        # Get all versions that appears Wordpress 3.x.x or 4.x.x
        _version = set(x[1] for x in re.findall(REGEX_FIND_VER, curr_content_2)
                       if x[1].startswith("3") or x[1].startswith("4"))

        # Get the major version of the plugin
        _tmp = "0.0.0"
        _last_version_tuple = tuple(last_version.split("."))
        for _ver in _version:
            _curr_ver = tuple(list(_version)[0].split("."))
            _tmp_tuple = tuple(_tmp.split("."))
            if _tmp_tuple < _curr_ver < _last_version_tuple:
                _tmp = _ver

        if _tmp != "0.0.0":
            cur_ver_2 = _tmp

    # --------------------------------------------------------------------------
    # Match versions of the different methods
    # --------------------------------------------------------------------------
    return_current_version = "unknown"
    if curr_ver is None and cur_ver_2 is None:
        return_current_version = "unknown"
    elif curr_ver is None and cur_ver_2 is not None:
        return_current_version = cur_ver_2
    elif curr_ver is not None and cur_ver_2 is None:
        return_current_version = curr_ver
    elif curr_ver is not None and cur_ver_2 is not None:
        if curr_ver != cur_ver_2:
            return_current_version = cur_ver_2
        else:
            return_current_version = curr_ver
    else:
        return_current_version = "unknown"

    # If Current version not found
    if return_current_version == "unknown":
        for url_pre, regex in url_version.items():
            # URL to find wordpress version
            url_current_version = urljoin(url, url_pre)
            _, _, current_version_content = yield from download(url_current_version, auto_redirect=False)

            # Find the version
            if current_version_content is not None:
                tmp_version = re.search(regex, current_version_content)

                if tmp_version is not None:
                    return_current_version = tmp_version.group(2)
                    break  # Found -> stop search

    # Get wordpress vulnerabilities
    return PlecostWordPressInfo(current_version=return_current_version,
                                last_version=last_version,
                                vulnerabilities=get_wordpress_vulnerabilities(return_current_version, db))


# ----------------------------------------------------------------------
def get_wordpress_vulnerabilities(current_version, db):
    """
    Get CVEs associated to the installed Wordpress version

    :param current_version: PlecostWordPressInfo instance
    :type current_version: PlecostWordPressInfo

    :param db: cve database instance
    :type db: DB

    :return: list with CVEs
    :rtype: list(str)

    """
    if not isinstance(db, DB):
        raise TypeError("Expected DB, got '%s' instead" % type(db))

    if current_version is "unknown" or \
            not current_version:
        return []

    # Get CVE list
    return list(set(x for x in db.query_wordpress(current_version)))


__all__ = ("is_remote_a_wordpress", "get_wordpress_version")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import csv
import lxml.html

from urllib.error import URLError
from urllib.request import urlopen

from os.path import join

from ..utils import colorize, get_data_folder, update_progress


# --------------------------------------------------------------------------
def update_plugins(log):
    """
    Update data information.

    :param log: Log function as format: function(Message, Level)
    :type log: function(str, int)
    """
    # --------------------------------------------------------------------------
    # Config and vars
    # --------------------------------------------------------------------------
    wp_plugins_url = "http://wordpress.org/plugins/browse/popular/page/%s/"
    max_plugins = 1400

    file_out = join(get_data_folder(), "plugin_list_huge.txt")

    log("[*] Preparing for update plugins...\n")

    with open(file_out, "w") as out:

        already_processed = []
        already_processed_append = already_processed.append

        csv_file = csv.writer(out)

        total_plugins = 1
        searching = True

        # Looking for 85 * 14 (per page) = 1190  plugins
        for i in update_progress(range(1, 85),
                                 prefix_text="[*] Downloading plugins (slow): "):
            if searching is False:
                break

            # 6 tries for each request
            for x in range(1, 6):
                try:
                    url = wp_plugins_url % i
                    wpage = urlopen(url).read()
                    break
                except URLError as e:
                    log("[%s] %s" % (
                        colorize("!", "red"),
                        colorize("Error while getting URL: %s. Attempt %s.\n" % (url, x))
                    ))
                    # sleep(random())

                    # Maximum attempt reached
                    if x == 6:
                        log("[%s] %s" % (
                            colorize("!!", "red"),
                            colorize("Maximum time exceeded"),
                        ), 0)
                        return
                    else:
                        continue

            # Parse
            parsed_main = lxml.html.fromstring(wpage)

            for section in parsed_main.xpath('//main/article'):
                plugin_info = section.xpath(".//h2/a")[0]
                plugin_url = plugin_info.attrib.get("href")
                plugin_name = plugin_info.text

                if not plugin_name:
                    plugin_name = plugin_url

                #
                # Get plugins details
                #
                plugin_page = urlopen(plugin_url).read()

                plugin_parsed = lxml.html.fromstring(plugin_page)

                plugin_version = plugin_parsed.xpath("//div[contains(@class, 'plugin-meta')]/ul/li/strong")
                if plugin_version:
                    plugin_version = plugin_version[0].text
                else:
                    plugin_version = None

                # --------------------------------------------------------------------------
                # We have all information to continue?
                # --------------------------------------------------------------------------
                if plugin_url is None or plugin_version is None:
                    log("[%s] Not enough information to store plugin for:\n%s\n" %
                        (
                            colorize("ii", "red"),
                            plugin_info
                        ), 2)
                    continue

                # Report status
                log("    |-- %s - Processing plugin: %s\n" %
                    (
                        colorize(total_plugins),
                        plugin_url
                    ), log_level=1)

                # Write to file
                plugin_url_store = plugin_url.replace("https://wordpress.org/plugins/", "")[0:-1]
                try:
                    csv_file.writerow([plugin_url_store,
                                       plugin_name,
                                       plugin_version])
                except UnicodeEncodeError:
                    csv_file.writerow([plugin_url_store,
                                       plugin_url_store,
                                       plugin_version])

                # Save plugin
                already_processed_append(plugin_url)

                # Maximum number of plugins reached?
                total_plugins += 1

                if total_plugins >= max_plugins:
                    searching = False
                    break

    # Creates split files
    with open(file_out, "r") as all_plugins, \
            open(join(get_data_folder(), "plugin_list_10.txt"), 'w') as f_10, \
            open(join(get_data_folder(), "plugin_list_50.txt"), 'w') as f_50, \
            open(join(get_data_folder(), "plugin_list_100.txt"), 'w') as f_100, \
            open(join(get_data_folder(), "plugin_list_250.txt"), 'w') as f_250, \
            open(join(get_data_folder(), "plugin_list_1000.txt"), 'w') as f_1000:

        for i, line in enumerate(all_plugins.readlines(), start=1):

            _line = line

            if i < 11:
                f_10.write(_line)
            if i < 50:
                f_50.write(_line)
            if i < 100:
                f_100.write(_line)
            if i < 250:
                f_250.write(_line)
            if i < 1000:
                f_1000.write(_line)

    log("\n[*] Oks!\n")




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


"""
This file contains some orphan functions
"""
import urllib
import aiohttp
import asyncio

import os.path as op
from urllib.parse import urljoin
from random import choice, randint
from difflib import SequenceMatcher
from string import ascii_letters, digits

try:
    from termcolor import colored
except ImportError:
    def colored(text, color):
        return text

__all__ = ["colorize", "generate_error_page", "get_diff_ratio", "get_data_folder", "check_redirects"]


# ----------------------------------------------------------------------
def log(message, log_level=0, current_log_level=None):
    """
    Auxiliary function to use as log level

    :param current_log_level: Log level selected at the moment of running the program
    :type current_log_level: int

    :param message: Message to display
    :type message: basestring

    :param log_level: log level: 0-4
    :type log_level: int
    """
    from sys import stdout
    from os import environ

    if current_log_level is None:
        try:
            _current_log_level = int(environ["PLECOST_LOG_LEVEL"])
        except KeyError:
            _current_log_level = 0
    else:
        _current_log_level = current_log_level

    if log_level <= _current_log_level:
        print(message, end='')
        stdout.flush()


# ----------------------------------------------------------------------
def colorize(text, color="yellow", activate=True):
    """
    This function return the text in a indicated color, if color is activate.

    :param text: Text to colorize
    :type text: basestring

    :param color: Color name
    :type color: basestring

    :param activate: boolean value that indicates if color is enabled.
    :type activate: bool

    :return: string with colored (if activated) text.
    :rtype: str
    """
    if activate:
        return colored(text, color)


# ----------------------------------------------------------------------
def generate_error_page(url):
    """
    Takes an URL to an existing document and generates a random URL
    to a nonexisting document, to trigger a server error.

    Example:

    >>> from libs.utils import generate_error_page
    >>> generate_error_page("http://www.site.com/index.php")
    'http://www.site.com/index.php.19ds_8vjX'

    :param url: Original URL.
    :type  url: str

    :return: Generated URL.
    :rtype: str
    """
    if not isinstance(url, str):
        raise TypeError("Expected basestring, got '%s' instead" % type(url))

    # Get random path
    random_path = "".join(choice(ascii_letters + digits) for _ in range(randint(5, 20)))

    return "%s/%s" % (url, random_path)


# ------------------------------------------------------------------------------
def get_diff_ratio(text1, text2):
    """
    Compare two texts and return a floating point value between 0 and 1 with
    the difference ratio, with 0 being absolutely different and 1 being
    absolutely equal - the more similar the two texts are, the closer the ratio
    will be to 1.

    ..note:
        This function was taken from Golismero project: http://github.com/golismero/golismero

    :param text1: First text to compare.
    :type text1: basestring

    :param text2: Second text to compare.
    :type text2: basestring

    :returns: Floating point value between 0 and 1.
    :rtype: float
    """

    # Solve some trivial type errors (like using None).
    if not text1:
        text1 = ""
    if not text2:
        text2 = ""

    # Trivial case, the two texts are identical.
    if text1 == text2:
        return 1.0

    # Use the difflib sequence matcher to calculate the ratio.
    m = SequenceMatcher(a=text1, b=text2)
    return m.ratio()


# ----------------------------------------------------------------------
def get_data_folder():
    """
    Return path of resources folder.

    :return: path of resources folder.
    :rtype: str
    """
    return op.abspath(op.join(op.dirname(__file__), "..", "resources"))


# ----------------------------------------------------------------------
def update_progress(values, print_function=None, prefix_text="", bar_len=40):
    """
    Creates a process bar in ASCII


    >>> for x in update_progress(range(1, 65), prefix_text="Prefix text: "):
    Prefix text: [########] 20.31%

    :param values: list or generator with items to process
    :type values: list

    :param print_function: function used to display information. By default is buildin 'print'.
    :type print_function: function

    :param prefix_text: Text to write before bar.
    :type prefix_text: str

    :param bar_len: number of characters used into bar.
    :type bar_len: int

    """
    _values = list(values)
    _len_values = len(_values)
    _var_len = bar_len
    _print = print_function or print

    for i, x in enumerate(_values, start=1):
        _percent = (i/_len_values)
        _percent_fix = int((i/_len_values) * _var_len)
        _print('\r{0} [#{1}] {2:.2f}%'.format(prefix_text, '#'*_percent_fix, _percent*100), end='')
        yield x


# ------------------------------------------------------------------------------
@asyncio.coroutine
def download(url,
             max_redirect=2,
             loop=None,
             session=None,
             method="get",
             get_content=True,
             auto_redirect=True):
    """
    Download a web page content.

    :param url: path where to get information.
    :type url: basestring

    :param max_tries: maximum number of retries for each page
    :type max_tries: int

    :param connector: HTTPConnection instance
    :type connector: `HTTPConnection`

    :param loop: Event loop object
    :type loop: loop

    :param method: HTTP method to use
    :type method: str

    :param get_content: boolean value that indicates if must download content or not
    :type get_content: bool

    :return: Web page content as a tuple: (http_header, status, basestring)
    :rtype: (dict, int, str)
    """
    _loop = loop or asyncio.get_event_loop()

    # session = session or aiohttp.ClientSession(loop=_loop)

    if max_redirect < 0:
        return None, None, None

    # with aiohttp.Timeout(timeout=5):
    response = yield from session.request(
        method,
        url,
        allow_redirects=False)

    ret_status = None
    ret_headers = None
    ret_content = None

    if response.status in (300, 301, 302, 303, 307):
        location = response.headers.get('location')
        next_url = urllib.parse.urljoin(url, location)
        if max_redirect > 0:
            log('\n[!] redirect to %r from %r\n' % (next_url, url),
                log_level=1)
            if auto_redirect is True:
                return _loop.run_until_complete(download(next_url,
                                                         max_redirect=(max_redirect-1)))
            else:
                ret_headers, ret_status, ret_content = response.headers, response.status, None
        else:
            log('\n[!] redirect limit reached for %r from %r\n' % (next_url, url), log_level=2)

            ret_headers, ret_status, ret_content = response.headers, response.status, None
    else:
        content = None

        if get_content:
            content = (yield from response.read()).decode(errors="ignore")

        ret_headers, ret_status, ret_content = response.headers, response.status, content

    return ret_headers, ret_status, ret_content


# --------------------------------------------------------------------------
class ConcurrentDownloader:
    """Get a list of URLs. Download their content and call user defined
    action """

    # ----------------------------------------------------------------------
    def __init__(self,
                 process_url_content,
                 session,
                 max_redirects=2,
                 max_tasks=10,
                 loop=None):
        """
        :param process_url_content: function to process URL content, after it is downloaded
        :type process_url_content: function

        :param max_redirects: maximum number of redirects
        :type max_redirects: int

        :param max_tries: maximum number of HTTP retries.
        :type max_tries: int

        :param max_tasks: maximum number of concurrent tasks
        :type max_tasks: int

        :param loop: optional event loop object
        :type loop: loop

        :param connector: aioTCPConnector object
        :type connector: aiohttp.TCPConnector


        >>> import asyncio
        >>> display=lambda x: print(x)
        >>> loop = asyncio.get_event_loop()
        >>> v = ConcurrentDownloader(url_base="http://myhost.com", process_url_content=display)
        >>> loop.run_until_complete(v.run())
        """
        self.session = session
        self.max_redirects = max_redirects
        self.process_url_function = process_url_content or (lambda x: None)
        self.max_tasks = max_tasks
        self.loop = loop or asyncio.get_event_loop()
        self.q = asyncio.Queue(loop=self.loop)
        self.__results = []
        self.__results_append = self.results.append

    # ----------------------------------------------------------------------
    @property
    def results(self):
        return self.__results

    # ----------------------------------------------------------------------
    @asyncio.coroutine
    def _work(self):
        while True:
            # Get an URL to process
            url = yield from self.q.get()

            # Download content
            log("\n    |- Trying: %s" % colorize(url, "yellow"), log_level=1)

            headers, status, content = yield from download(url,
                                                           session=self.session,
                                                           max_redirect=self.max_redirects,
                                                           loop=self.loop)

            # Processing response
            _r = self.process_url_function(url, headers, status, content)
            if _r is not None:
                self.__results_append(_r)

            del headers, status, content

            self.q.task_done()

    # --------------------------------------------------------------------------
    #
    # Public method
    #
    # ----------------------------------------------------------------------
    def close(self):
        """Close resources."""
        self.connector.close()

    # --------------------------------------------------------------------------
    @asyncio.coroutine
    def run(self):
        """
        Start the analyzer daemon.
        """

        # Add workers
        workers = [asyncio.Task(self._work(), loop=self.loop)
                   for _ in range(self.max_tasks)]

        # Wait content of workers ends
        yield from self.q.join()

        for w in workers:
            w.cancel()

    # ----------------------------------------------------------------------
    def add_url(self, url):
        """
        Add a URL to queue to analyze

        :param url: URL to store
        :type url: str
        """
        self.q.put_nowait(url)

    # ----------------------------------------------------------------------
    def add_url_list(self, urls):
        """
        Add an URL list to processing

        :param urls: list with URLs
        :type urls: str

        """
        for x in urls:
            self.q.put_nowait(x)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

"""
This file contains Plecost exceptions
"""


# --------------------------------------------------------------------------
class PlecostWordListNotFound(Exception):
    """Word list not found"""


# --------------------------------------------------------------------------
class PlecostTargetNotAvailable(Exception):
    """Impossible to connect to the target"""


# --------------------------------------------------------------------------
class PlecostNotWordPressFound(Exception):
    """Exception when not a valid WordPress installation found"""


# --------------------------------------------------------------------------
class PlecostInvalidReportFormat(Exception):
    """Requested report file formant is not available."""
